#!/bin/bash
# installiert das MIYOplugin zur Anbindung des MIYO-Bewaesserungssystems an Hausautomatisierungssysteme
# Oliver Engel, 18.05.2019; 26.10.2019; 31.03.2020
# miyoplugin.phantasoft.de
#
# enthaelt miyoplugin.py und miyoplugin.service sowie generic-MIYOplugin-install.sh (dieses Script)
#
# Vorarbeiten:
# Verzeichnis erzeugen
# sudo mkdir /opt/MIYOplugin
#
# in das erzeugte Verzeichnis wechseln
# cd /opt/MIYOplugin
#
# aktuelle Version des Plugins per wget holen
# wget -N http://miyoplugin.phantasoft.de/files/generic-MIYOplugin.zip
#
# ZIP-File entpacken
# unzip generic-MIYOplugin.zip
#
# Execute-Recht fuer generic-MIYOplugin-install.sh (dieses Script) einraeumen
# chmod u+x generic-MIYOplugin-install.sh
#
# generic-MIYOplugin-install.sh (dieses Script) ausführen
# sudo ./generic-MIYOplugin-install.sh --install
#
PRGNAME=MIYOplugin
PRGVER=v0.05
MYDIR=`pwd`
REPLY=n
ISTR="+++ $PRGNAME +++"

clear
echo
echo Phantasoft install-Script for $PRGNAME $PRGVER
echo ----------------------------------------------
echo

if [ "$1" = "-update" ] || [ "$1" = "--update" ] || [ "$1" = "-upgrade" ] || [ "$1" = "--upgrade" ]; then
  echo $ISTR going to update MIYOplugin

  echo $ISTR get new MIYOplugin from the web
  wget -N miyoplugin.phantasoft.de/files/generic-MIYOplugin.zip

  echo $ISTR save the current configuration
  sudo -u "${SUDO_USER}" cp -v miyoplugin.conf miyoplugin.save

  echo $ISTR unzipping the new file
  sudo -u "${SUDO_USER}" unzip -o generic-MIYOplugin.zip

  echo $ISTR recover saved Config
  sudo -u "${SUDO_USER}" cp -v miyoplugin.conf miyoplugin.new
  sudo -u "${SUDO_USER}" cp -v miyoplugin.save miyoplugin.conf

  echo $ISTR set executable to miyoplugin.py
  chmod 711 -v miyoplugin.py
  chmod 711 -v generic-MIYOplugin-install.sh

  echo $ISTR restarting MIYOplugin-service if running
  systemctl is-active --quiet service && systemctl restart miyoplugin

  echo
  echo $ISTR upgrade-installation complete
elif [ "$1" = "-uninstall" ] || [ "$1" = "--uninstall" ]; then
  # uninstall service
  echo $ISTR uninstalling $PRGNAME $PRGVER
  echo

  echo $ISTR disable system-service miyoplugin
  systemctl stop miyoplugin
  systemctl disable miyoplugin
  systemctl daemon-reload

  echo $ISTR remove system-service miyoplugin from auto-start
  rm -f /etc/systemd/system/miyoplugin.service

  echo
  echo $ISTR system-service miyoplugin should be uninstalled
  echo $ISTR all files are still in $MYDIR
  echo
elif [ "$1" = "-install" ] || [ "$1" = "--install" ]; then
  echo $ISTR install and configure MIYOplugin
  until [[ $REPLY =~ ^[YyJj]$ ]]; do
    echo
    # Target-IP abfragen
    read -p "$ISTR ip address of target system: " LOX_IP
    # Target-Port abfragen
    read -p "$ISTR udp port on target system: " LOX_PORT
    # lokalen Port abfragen
    read -p "$ISTR udp port on local system: " LB_PORT
    echo
    read -p "$ISTR are these settings ok? (Y/N) " -n 1 -r
    echo    # (optional) move to a new line
  done

  # erforderliche Pakete nachinstallieren
  read -t 1 -n 10000 discard
  echo
  echo $ISTR we will now install Python3 via apt
  apt-get update && apt-get upgrade
  apt-get -y install python3 python3-pip

  # erforderliche Python-Pakete nachinstallieren
  echo
  echo $ISTR now we have to install the python-libs
  echo $ISTR requests, websocket and websocket-client
  pip3 install requests websocket websocket-client

  # Dateirechte anpassen
  echo
  echo $ISTR set executable to miyoplugin.py
  chmod 711 -v miyoplugin.py

  # Pfadnamen fuer Log in der Config anpassen
  /bin/sed -i "s#REPLACEMIYOLOGDIR#$MYDIR#" miyoplugin.conf

  # Config-File erzeugen/aktualisieren (fragt MIYO-IP, API-Key und Circuits ab)
  read -t 1 -n 10000 discard
  echo
  echo $ISTR configuring MIYOplugin - please press the button on the Cube!
  read -p "$ISTR press ENTER after you have triggered the Cube-button"
  createConfig=`./miyoplugin.py -createConfig $LOX_IP $LOX_PORT $LB_PORT False False`

  # Rechte der Log-Dateien ggf. anpassen
  echo
  echo $ISTR change owner of *.log to "${SUDO_USER}:${SUDO_USER}"
  chown -v "${SUDO_USER}:${SUDO_USER}" *.log
 
  # Dienst einrichten und starten
  # ersetze REPLACEMIYODATADIR mit aktuellem Verzeichnis
  echo
  echo $ISTR installing system-service via systemd
  cp -f miyoplugin.service miyoplugin.service.tmp
  /bin/sed -i "s#REPLACEMIYODATADIR#$MYDIR#" miyoplugin.service.tmp
  cp -f miyoplugin.service.tmp /etc/systemd/system/miyoplugin.service
  chown -v "${SUDO_USER}:${SUDO_USER}" miyoplugin.service.tmp
  systemctl import-environment
  systemctl daemon-reload && systemctl enable miyoplugin && systemctl start miyoplugin --no-block

  # fertig
  # jetzt ggf. mit einem Editor noch die fehlenden Settings nachtragen
  echo
  echo
  echo $ISTR everything should be ok now, MIYOplugin should be up and ready
  ps ax|grep miyoplugin|grep -v grep
  echo
  echo $ISTR current configuration is:
  cat miyoplugin.conf
  udpok=`./miyoplugin.py -checkLBPort $LB_PORT`
  echo check UDP Port $LB_PORT if usable: $udpok
  echo
  echo $createConfig
elif [ "$1" = "-repair" ] || [ "$1" = "--repair" ]; then
  echo $ISTR repair installation of MIYOplugin
  pip3 install --upgrade websocket-client requests
  echo
  echo $ISTR repairing installation complete
else
  echo you have to use parameter -install, -uninstall, -upgrade or -repair
  echo
  echo nothing done
fi
echo

